# -*- coding: utf-8 -*-
# Copyright 2009-2014, Red Hat, Inc.
# License: GPL-2.0+ <http://spdx.org/licenses/GPL-2.0+>
# See the LICENSE file for more details on Licensing

'''Unit tests for libtaskotron/koji_utils.py'''

import pytest
from dingus import Dingus

from libtaskotron import koji_utils
from libtaskotron import exceptions as exc
from libtaskotron import config

class TestKojiClient():
    def setup_method(self, method):
        self.ref_nvr = 'foo-1.2-3.fc99'
        self.ref_arch = 'noarch'
        self.ref_name = 'foo'
        self.ref_version = '1.2'
        self.ref_release = '3.fc99'
        self.ref_buildid = 123456
        self.ref_filename = "%s.%s.rpm" % (self.ref_nvr, self.ref_arch)

        self.ref_build = {'package_name': self.ref_name,
                          'version': self.ref_version,
                          'release': self.ref_release,
                          'id': self.ref_buildid}

        self.ref_rpms = [{'name': self.ref_name, 'version': self.ref_version,
                          'release': self.ref_release, 'nvr': self.ref_nvr,
                          'arch': self.ref_arch, 'build_id': self.ref_buildid,},
                    {'name': self.ref_name, 'version': self.ref_version,
                     'release': self.ref_release, 'nvr': self.ref_nvr,
                     'arch': 'src', 'build_id': self.ref_buildid, }]

    def test_rpm_to_build(self):
        stub_koji = Dingus(getBuild__returns = self.ref_build,
                           listRPMs__returns = self.ref_rpms,
                           getRPM__returns = self.ref_rpms[0])

        test_koji = koji_utils.KojiClient(stub_koji)
        outcome = test_koji.rpm_to_build(self.ref_filename)

        assert outcome is self.ref_build
        assert len(stub_koji.calls) == 3

        # test prefetch
        outcome = test_koji.rpm_to_build(self.ref_filename)
        assert len(stub_koji.calls) == 3

    def test_rpm_to_build_exceptions(self):
        stub_koji = Dingus(getRPM__returns = None)
        test_koji = koji_utils.KojiClient(stub_koji)

        with pytest.raises(exc.TaskotronRemoteError):
            test_koji.rpm_to_build(self.ref_filename)


        stub_koji = Dingus(getBuild__returns = None,
                           listRPMs__returns = self.ref_rpms,
                           getRPM__returns = self.ref_rpms[0])

        test_koji = koji_utils.KojiClient(stub_koji)
        with pytest.raises(exc.TaskotronRemoteError):
            test_koji.rpm_to_build(self.ref_filename)


    def test_get_noarch_rpmurls_from_nvr(self):
        stub_koji = Dingus(getBuild__returns = self.ref_build,
                           listRPMs__returns = self.ref_rpms)

        test_koji = koji_utils.KojiClient(stub_koji)

        test_urls = test_koji.nvr_to_urls(self.ref_nvr)

        koji_baseurl = config.get_config().pkg_url
        ref_urls = ['%s/%s/%s/%s/%s/%s.%s.rpm' % (koji_baseurl, self.ref_name,
                        self.ref_version, self.ref_release, self.ref_arch,
                        self.ref_nvr, self.ref_arch),
                    '%s/%s/%s/%s/src/%s.src.rpm' % (koji_baseurl, self.ref_name,
                        self.ref_version, self.ref_release, self.ref_nvr),
                    ]

        assert test_urls == ref_urls

    def should_i386_rpmurls_query_i686(self):
        self.ref_arch = 'i386'

        stub_koji = Dingus(getBuild__returns = self.ref_build)

        test_koji = koji_utils.KojiClient(stub_koji)

        # this isn't setup to actually return urls since we don't care about
        # that for this test
        # it should throw an exception in this case.
        try:
            test_koji.nvr_to_urls(self.ref_nvr, arches = [self.ref_arch])
        except exc.TaskotronRemoteError:
            pass

        listrpm_calls = stub_koji.calls('listRPMs')
        requested_arches = listrpm_calls[0][2]['arches']

        assert 'i686' in requested_arches
        assert 'i386' in requested_arches

    def should_throw_exception_norpms(self):
        stub_koji = Dingus(getBuild__returns = self.ref_build)

        test_koji = koji_utils.KojiClient(stub_koji)

        with pytest.raises(exc.TaskotronRemoteError):
            test_koji.nvr_to_urls(self.ref_nvr, arches = [self.ref_arch])

    def test_handle_norpms_in_build(self, tmpdir):
        """ This tests to make sure that missing rpms in a build are handled
        gracefully during download so that execution isn't stopped when a build
        is missing an rpm """

        rpmdir = tmpdir.mkdir("rpmdownload")

        stub_koji = Dingus(getBuild__returns = self.ref_build)

        test_koji = koji_utils.KojiClient(stub_koji)

        test_koji.get_nvr_rpms(self.ref_nvr, str(rpmdir), arches = [self.ref_arch])


class TestGetENVR(object):

    def test_epoch(self):
        '''Epoch included in build'''
        build = {'nvr': 'foo-1.2-3.fc20',
                 'epoch': 1,
                }
        assert koji_utils.getNEVR(build) == 'foo-1:1.2-3.fc20'

    def test_no_epoch(self):
        '''Epoch not included in build'''
        build = {'nvr': 'foo-1.2-3.fc20',
                 'epoch': None,
                }
        assert koji_utils.getNEVR(build) == 'foo-1.2-3.fc20'

    def test_raise(self):
        '''Invalid input param'''
        with pytest.raises(exc.TaskotronValueError):
            koji_utils.getNEVR('foo-1.2-3.fc20')
